@@ -17,7 +17,7 @@
   +----------------------------------------------------------------------+
 */
 
-/* $Id: memcache.c,v 1.83.2.36 2009/02/17 22:39:22 mikl Exp $ */
+/* $Id: memcache.c 310129 2011-04-11 04:44:27Z hradtke $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -75,6 +75,7 @@ static zend_function_entry php_memcache_pool_class_functions[] = {
 	PHP_FALIAS(setserverparams,			memcache_set_server_params,			NULL)
 	PHP_FALIAS(setfailurecallback,		memcache_set_failure_callback,		NULL)
 	PHP_FALIAS(getserverstatus,			memcache_get_server_status,			NULL)
+	PHP_NAMED_FE(findserver,			zif_memcache_pool_findserver,		NULL)
 	PHP_FALIAS(getversion,				memcache_get_version,				NULL)
 	PHP_FALIAS(add,						memcache_add,						NULL)
 	PHP_FALIAS(set,						memcache_set,						NULL)
@@ -264,11 +265,6 @@ PHP_INI_BEGIN()
 PHP_INI_END()
 /* }}} */
 
-/* {{{ macros */
-#define MMC_PREPARE_KEY(key, key_len) \
-	php_strtr(key, key_len, "\t\r\n ", "____", 4); \
-/* }}} */
-
 /* {{{ internal function protos */
 static void _mmc_pool_list_dtor(zend_rsrc_list_entry * TSRMLS_DC);
 static void _mmc_server_list_dtor(zend_rsrc_list_entry * TSRMLS_DC);
@@ -336,7 +332,7 @@ PHP_MINFO_FUNCTION(memcache)
 	php_info_print_table_start();
 	php_info_print_table_header(2, "memcache support", "enabled");
 	php_info_print_table_row(2, "Version", PHP_MEMCACHE_VERSION);
-	php_info_print_table_row(2, "Revision", "$Revision: 1.83.2.36 $");
+	php_info_print_table_row(2, "Revision", "$Revision: 310129 $");
 	php_info_print_table_end();
 
 	DISPLAY_INI_ENTRIES();
@@ -400,7 +396,8 @@ int mmc_stored_handler(mmc_t *mmc, mmc_request_t *request, int response, const c
 	}
 
 	/* return FALSE or catch memory errors without failover */
-	if (response == MMC_RESPONSE_EXISTS || response == MMC_RESPONSE_OUT_OF_MEMORY || response == MMC_RESPONSE_TOO_LARGE) {
+	if (response == MMC_RESPONSE_EXISTS || response == MMC_RESPONSE_OUT_OF_MEMORY || response == MMC_RESPONSE_TOO_LARGE 
+			|| response == MMC_RESPONSE_CLIENT_ERROR) {
 		ZVAL_FALSE(result);
 
 		if (response != MMC_RESPONSE_EXISTS) {
@@ -546,7 +543,7 @@ int mmc_numeric_response_handler(mmc_t *mmc, mmc_request_t *request, int respons
 		return MMC_REQUEST_DONE;
 	}
 
-	if (response == MMC_RESPONSE_NOT_FOUND) {
+	if (response == MMC_RESPONSE_NOT_FOUND || response == MMC_RESPONSE_CLIENT_ERROR) {
 		if (Z_TYPE_P(result) == IS_ARRAY) {
 			add_assoc_bool_ex(result, request->key, request->key_len + 1, 0);
 		}
@@ -554,6 +551,13 @@ int mmc_numeric_response_handler(mmc_t *mmc, mmc_request_t *request, int respons
 			ZVAL_FALSE(result);
 		}
 
+		if (response != MMC_RESPONSE_NOT_FOUND) {
+			php_error_docref(NULL TSRMLS_CC, 
+					E_NOTICE, "Server %s (tcp %d, udp %d) failed with: %s (%d)",
+					mmc->host, mmc->tcp.port, 
+					mmc->udp.port, message, response);
+		}
+
 		return MMC_REQUEST_DONE;
 	}
 
@@ -642,7 +646,7 @@ static void php_mmc_numeric(INTERNAL_FUNCTION_PARAMETERS, int deleted, int inver
 			}
 
 			if (deleted) {
-				pool->protocol->delete(request, request->key, request->key_len, value);
+				pool->protocol->delete(request, request->key, request->key_len, exptime);
 			}
 			else {
 				pool->protocol->mutate(request, *key, request->key, request->key_len, invert ? -value : value, defval, defval_used, exptime);
@@ -676,7 +680,7 @@ static void php_mmc_numeric(INTERNAL_FUNCTION_PARAMETERS, int deleted, int inver
 		}
 
 		if (deleted) {
-			pool->protocol->delete(request, request->key, request->key_len, value);
+			pool->protocol->delete(request, request->key, request->key_len, exptime);
 		}
 		else {
 			pool->protocol->mutate(request, keys, request->key, request->key_len, invert ? -value : value, defval, defval_used, exptime);
@@ -762,7 +766,7 @@ static mmc_t *php_mmc_pool_addserver(
 	mmc_t *mmc;
 	int list_id, resource_type;
 
-	if (weight < 0) {
+	if (weight < 1) {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "weight must be a positive integer");
 		return NULL;
 	}
@@ -1148,6 +1152,39 @@ PHP_NAMED_FUNCTION(zif_memcache_pool_addserver)
 }
 /* }}} */
 
+/* {{{ proto string MemcachePool::findServer(string key)
+	Returns the server corresponding to a key
+*/
+PHP_NAMED_FUNCTION(zif_memcache_pool_findserver)
+{
+	zval *mmc_object = getThis();
+	mmc_pool_t *pool;
+	mmc_t *mmc;
+	
+	zval *zkey;
+	char key[MMC_MAX_KEY_LEN + 1];
+	unsigned int key_len;
+	char *hostname;
+
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "z", &zkey) == FAILURE) {
+		return;
+	}
+	
+	if (!mmc_get_pool(mmc_object, &pool TSRMLS_CC) || !pool->num_servers) {
+		RETURN_FALSE;
+	}
+
+	if (mmc_prepare_key(zkey, key, &key_len) != MMC_OK) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Invalid key");
+		RETURN_FALSE;
+	}
+	
+	mmc = mmc_pool_find(pool, key, key_len TSRMLS_CC);
+	spprintf(&hostname, 0, "%s:%d", mmc->host, mmc->tcp.port);	
+	RETURN_STRING(hostname, 0);
+}
+/* }}} */
+
 /* {{{ proto bool memcache_add_server(string host [, int port [, bool persistent [, int weight [, double timeout [, int retry_interval [, bool status [, callback failure_callback ] ] ] ] ] ] ])
    Adds a connection to the pool. The order in which this function is called is significant */
 PHP_FUNCTION(memcache_add_server)
@@ -1890,6 +1927,16 @@ static int mmc_flush_handler(mmc_t *mmc, mmc_request_t *request, int response, c
 		return MMC_REQUEST_DONE;
 	}
 
+	if (response == MMC_RESPONSE_CLIENT_ERROR) {
+		ZVAL_FALSE((zval *)param);
+		php_error_docref(NULL TSRMLS_CC, E_NOTICE, 
+				"Server %s (tcp %d, udp %d) failed with: %s (%d)",
+				mmc->host, mmc->tcp.port, 
+				mmc->udp.port, message, response);
+
+		return MMC_REQUEST_DONE;
+	}
+
 	return mmc_request_failure(mmc, request->io, message, message_len, 0 TSRMLS_CC);
 }
 /* }}} */
