@@ -17,7 +17,7 @@
   +----------------------------------------------------------------------+
 */
 
-/* $Id: memcache_ascii_protocol.c,v 1.1.2.7 2009/01/16 15:38:46 mikl Exp $ */
+/* $Id: memcache_ascii_protocol.c 310129 2011-04-11 04:44:27Z hradtke $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -84,11 +84,13 @@ static int mmc_request_check_response(const char *line, int line_len) /*
 	}
 	else if (
 		mmc_str_left(line, "ERROR", line_len, sizeof("ERROR")-1) ||
-		mmc_str_left(line, "SERVER_ERROR", line_len, sizeof("SERVER_ERROR")-1) ||
-		mmc_str_left(line, "CLIENT_ERROR", line_len, sizeof("CLIENT_ERROR")-1)) 
+		mmc_str_left(line, "SERVER_ERROR", line_len, sizeof("SERVER_ERROR")-1)) 
 	{
 		response = MMC_RESPONSE_ERROR;
 	}
+	else if (mmc_str_left(line, "CLIENT_ERROR", line_len, sizeof("CLIENT_ERROR")-1)) {
+		response = MMC_RESPONSE_CLIENT_ERROR;
+	}
 	else {
 		response = MMC_RESPONSE_UNKNOWN;
 	}
