@@ -17,7 +17,7 @@
   +----------------------------------------------------------------------+
 */
 
-/* $Id: memcache_pool.c,v 1.1.2.39 2009/02/22 16:02:43 mikl Exp $ */
+/* $Id: memcache_pool.c 310129 2011-04-11 04:44:27Z hradtke $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -363,6 +363,7 @@ int mmc_pack_value(mmc_pool_t *pool, mmc_buffer_t *buffer, zval *value, unsigned
 
 		case IS_LONG:
 			*flags |= MMC_TYPE_LONG;
+			*flags &= ~MMC_COMPRESSED;
 			smart_str_append_long(&(buffer->value), Z_LVAL_P(value));
 			break;
 
@@ -370,12 +371,14 @@ int mmc_pack_value(mmc_pool_t *pool, mmc_buffer_t *buffer, zval *value, unsigned
 			char buf[256];
 			int len = snprintf(buf, 256, "%.14g", Z_DVAL_P(value));
 			*flags |= MMC_TYPE_DOUBLE;
+			*flags &= ~MMC_COMPRESSED;
 			smart_str_appendl(&(buffer->value), buf, len);
 			break;
 		}
 
 		case IS_BOOL:
 			*flags |= MMC_TYPE_BOOL;
+			*flags &= ~MMC_COMPRESSED;
 			smart_str_appendc(&(buffer->value), Z_BVAL_P(value) ? '1' : '0');
 			break;
 
@@ -777,6 +780,14 @@ static int mmc_server_connect(mmc_pool_t *pool, mmc_t *mmc, mmc_stream_t *io, in
 		io->read = mmc_stream_read_wrapper;
 		io->readline = mmc_stream_readline_wrapper;
 	}
+	
+#ifdef SO_NOSIGPIPE
+	/* Mac OS X doesn't have MSG_NOSIGNAL */
+	{
+		int optval = 1;
+		setsockopt(io->fd, SOL_SOCKET, SO_NOSIGPIPE, (void *)&optval, sizeof(optval));
+	}
+#endif
 
 	if (mmc->error != NULL) {
 		efree(mmc->error);
