@@ -17,7 +17,7 @@
   +----------------------------------------------------------------------+
 */
 
-/* $Id: memcache_pool.h,v 1.1.2.23 2009/02/17 22:39:22 mikl Exp $ */
+/* $Id: memcache_pool.h 310129 2011-04-11 04:44:27Z hradtke $ */
 
 #ifndef MEMCACHE_POOL_H
 #define MEMCACHE_POOL_H
@@ -26,6 +26,14 @@
 #include "config.h"
 #endif
 
+#ifdef HAVE_SYS_TYPES_H
+#include <sys/types.h>
+#endif
+
+#ifdef HAVE_SYS_SOCKET_H
+#include <sys/socket.h>
+#endif
+
 #include <stdint.h>
 #include <string.h>
 
@@ -41,6 +49,15 @@
 #define ZSTR_VAL(v) (v).s
 #endif
 
+/*
+ * Mac OS X has no MSG_NOSIGNAL but >= 10.2 comes with SO_NOSIGPIPE which is a setsockopt() option
+ * and not a send() parameter as MSG_NOSIGNAL. OpenBSD has none of the options so we need to ignore 
+ * SIGPIPE events
+ */
+#ifndef MSG_NOSIGNAL
+#define MSG_NOSIGNAL 0
+#endif /*MSG_NOSIGNAL*/
+
 /* use lowest byte for flags */
 #define MMC_SERIALIZED	0x0001
 #define MMC_COMPRESSED	0x0002
@@ -87,6 +104,7 @@
 #define MMC_RESPONSE_EXISTS			0x02			/* same as binary protocol */
 #define MMC_RESPONSE_TOO_LARGE		0x03			/* same as binary protocol */
 #define MMC_RESPONSE_NOT_STORED		0x05			/* same as binary protocol */
+#define MMC_RESPONSE_CLIENT_ERROR	0x06
 #define MMC_RESPONSE_UNKNOWN_CMD	0x81			/* same as binary protocol */
 #define MMC_RESPONSE_OUT_OF_MEMORY	0x82			/* same as binary protocol */
 
